#!/usr/bin/env python 
# -*- coding: utf-8 -*-
# @Time    : 2024/10/21 19:44
# @Author  : 兵
# @email    : 1747193328@qq.com
import traceback

import requests
from PySide6.QtCore import QUrl
from PySide6.QtGui import QDesktopServices
from PySide6.QtWidgets import QWidget
from qfluentwidgets import SettingCardGroup, HyperlinkCard, PrimaryPushSettingCard, ExpandLayout, MessageBox
from qfluentwidgets import FluentIcon as FIF

from NepTrainKit import utils
from NepTrainKit.core import MessageManager
from NepTrainKit.core.update import UpdateWoker
from NepTrainKit.version import HELP_URL, FEEDBACK_URL, __version__, YEAR, AUTHOR, RELEASES_URL

class SettingsWidget(QWidget):
    def __init__(self,parent):

        super().__init__(parent)
        self.setObjectName('SettingsWidget')
        self.expand_layout = ExpandLayout(self)
        self.setLayout(self.expand_layout)
        self.aboutGroup = SettingCardGroup("About", self)
        self.helpCard = HyperlinkCard(
            HELP_URL,
             'Open Help Page' ,
            FIF.HELP,
             'Help' ,
             'Discover new features and learn useful tips about NepTrainKit.' ,
            self.aboutGroup
        )
        self.feedbackCard = PrimaryPushSettingCard(
            "Submit Feedback",
            FIF.FEEDBACK,
            "Submit Feedback",

            'Help us improve NepTrainKit by providing feedback.',
            self.aboutGroup
        )
        self.aboutCard = PrimaryPushSettingCard(
            'Check for Updates',
            FIF.INFO,
            "About",
            'Copyright ©' + f" {YEAR}, {AUTHOR}. " +
            "Version" + f" {__version__}",
            self.aboutGroup
        )
        self.aboutGroup.addSettingCard(self.helpCard)
        self.aboutGroup.addSettingCard(self.feedbackCard)
        self.aboutGroup.addSettingCard(self.aboutCard)
        self.init_layout()
        self.init_signal()
    def init_layout(self):
        self.expand_layout.setSpacing(28)
        self.expand_layout.setContentsMargins(60, 10, 60, 0)
        self.expand_layout.addWidget(self.aboutGroup)
    def init_signal(self):
        self.aboutCard.clicked.connect(self.check_update)

        # self.aboutCard.clicked.connect(lambda: QDesktopServices.openUrl(QUrl(RELEASES_URL)))
        self.feedbackCard.clicked.connect(
            lambda: QDesktopServices.openUrl(QUrl(FEEDBACK_URL)))

    def check_update(self):
        UpdateWoker(self).check_update()



#!/usr/bin/env python 
# -*- coding: utf-8 -*-
# @Time    : 2024/10/18 22:12
# @Author  : 兵
# @email    : 1747193328@qq.com
from enum import Enum


import numpy as np
from PySide6.QtCore import Qt
from PySide6.QtGui import QAction, QIcon, QPen
from PySide6.QtWidgets import QToolBar
from pyqtgraph import ViewBox, PlotCurveItem

from NepTrainKit import utils
from NepTrainKit.core import Config
from NepTrainKit.core.custom_widget.dialog import GetIntMessageBox, SparseMessageBox
from NepTrainKit.core.io.select import farthest_point_sampling


class _Mode(str, Enum):
    NONE = ""
    PAN = "pan"
    ZOOM = "zoom rect"
    PEN="pen"
    def __str__(self):
        return self.value

    @property
    def _navigate_mode(self):
        return self.name if self is not _Mode.NONE else None





class NepDisplayGraphicsToolBar(QToolBar):


    def __init__(self,graph_widget, parent=None):
        super().__init__(parent)
        self._parent = parent
        self.graph_widget=graph_widget
        self.graph_widget.tool_bar=self
        self.graph_widget.currentPlotChanged.connect(self.reset_connect)
        self._actions={}
        self.init_actions()
        self.mode=_Mode.NONE
        if self.graph_widget.current_plot:
            self.current_plot=self.graph_widget.current_plot
            self.view_box = self.graph_widget.current_plot.getViewBox()
        else:
            self.current_plot=None
            self.view_box=None
    def init_actions(self):
        self.add_action("Reset View",QIcon(":/images/src/images/init.svg"),self.reset_view)
        pan_action=self.add_action("Pan View",QIcon(":/images/src/images/pan.svg"),self.pan)
        pan_action.setCheckable(True)



        find_max_action=self.add_action( "Find Max Error Point",
                                          QIcon(":/images/src/images/find_max.svg"),
                                          self.find_max_error_point)
        sparse_action=self.add_action( "Sparse samples",
                                          QIcon(":/images/src/images/sparse.svg"),
                                          self.sparse_point)


        pen_action=self.add_action("Mouse Selection",QIcon(":/images/src/images/pen.svg"),self.pen)
        pen_action.setCheckable(True)

        revoke_action=self.add_action("Undo",QIcon(":/images/src/images/revoke.svg"),self.revoke)

        delete_action=self.add_action("Delete Selected Items",QIcon(":/images/src/images/delete.svg"),self.delete)

    def add_action(self, name,icon,callback):
        action=QAction(QIcon(icon),name,self)
        action.triggered.connect(callback)
        self._actions[name]=action
        self.addAction(action)
        action.setToolTip(name)
        return action
    def reset_connect(self):
        """
        在选中工具栏的情况下  双击进行切换 会出现问题
        这里直接判断下
        :return:
        """



        if self.view_box is not None:

            del self.view_box.mousePressEvent
            del self.view_box.mouseMoveEvent
            del self.view_box.mouseReleaseEvent
            if self.mode==_Mode.PAN:
                self.pan(False)

            elif self.mode==_Mode.PEN:
                self.pen(False)
            self.mode = _Mode.NONE
            self._update_buttons_checked()
        self.current_plot=self.graph_widget.current_plot

        self.view_box=self.current_plot.getViewBox()

        self.view_box.mousePressEvent = self._mousePressEvent
        self.view_box.mouseMoveEvent = self._mouseMoveEvent
        self.view_box.mouseReleaseEvent = self._mouseReleaseEvent

    def reset_view(self):
        """重置视图"""
        if self.view_box is None:
            return False

        self.view_box.autoRange()

    def delete(self):
        self.graph_widget.delete()

    def revoke(self):
        self.graph_widget.revoke()

    def find_max_error_point(self):
        dataset = self.graph_widget.get_current_dataset()
        if dataset is None:
            return
        box= GetIntMessageBox(self.graph_widget._parent,"Please enter an integer N, it will find the top N structures with the largest errors")
        n = Config.getint("widget","max_error_value",10)
        box.intSpinBox.setValue(n)

        if not box.exec():
            return
        nmax= box.intSpinBox.value()
        Config.set("widget","max_error_value",nmax)
        index= (dataset.get_max_error_index(nmax))

        self.graph_widget.select_index(index,False)

    def sparse_point(self):
        if  self.graph_widget.dataset is None:
            return
        box= SparseMessageBox(self.graph_widget._parent,"Please specify the maximum number of structures and minimum distance")
        n_samples = Config.getint("widget","sparse_num_value",10)
        distance = Config.getfloat("widget","sparse_distance_value",0.01)

        box.intSpinBox.setValue(n_samples)
        box.doubleSpinBox.setValue(distance)

        if not box.exec():
            return
        n_samples= box.intSpinBox.value()
        distance= box.doubleSpinBox.value()

        Config.set("widget","sparse_num_value",n_samples)
        Config.set("widget","sparse_distance_value",distance)

        dataset = self.graph_widget.dataset.descriptor
        indices_to_remove = farthest_point_sampling(dataset.now_data,n_samples=n_samples,min_dist=distance)

        # 获取所有索引（从 0 到 len(arr)-1）
        all_indices = np.arange(dataset.now_data.shape[0])

        # 使用 setdiff1d 获取不在 indices_to_remove 中的索引
        remaining_indices = np.setdiff1d(all_indices, indices_to_remove)
        structures = dataset.group_array[remaining_indices]
        self.graph_widget.select_index(structures.tolist(),False)

    def pan(self, checked):
        """切换平移模式"""
        if self.view_box is None:
            self._update_buttons_checked()
            return False

        if checked:
            self.mode = _Mode.PAN

            self.view_box.setMouseEnabled(True, True)

            self.view_box.setMouseMode(ViewBox.PanMode)  # 启用平移模式
        else:
            self.mode = _Mode.NONE

            self.view_box.setMouseEnabled(False, False)
        self._update_buttons_checked()



    def pen(self, checked):
        if self.view_box is None:
            self._update_buttons_checked()

            return False

        if checked:
            self.mode = _Mode.PEN
            # 初始化鼠标状态和轨迹数据
            self.is_drawing = False
            self.x_data = []
            self.y_data = []

        else:
            self.mode = _Mode.NONE
        self._update_buttons_checked()
    def _update_buttons_checked(self,):

        # sync button checkstates to match active mode
        if 'Pan View' in self._actions:
            self._actions['Pan View'].setChecked(self.mode.name == 'PAN')

        if 'Mouse Selection' in self._actions:
            self._actions['Mouse Selection'].setChecked(self.mode.name == 'PEN')

    def _mousePressEvent(self, event):
        """鼠标按下时开始绘制"""
        if self.mode !=_Mode.PEN:
            return ViewBox.mousePressEvent(self.view_box,event)
        if event.button() == Qt.MouseButton.LeftButton or event.button() == Qt.MouseButton.RightButton:
            self.is_drawing = True
            self.x_data.clear()  # 清空之前的轨迹数据
            self.y_data.clear()  # 清空之前的轨迹数据
            self.curve = self.current_plot.plot([], [], pen='r')

            self.curve.setData([], [])  # 清空绘制线条，避免对角线
            # 创建一个单独的 QGraphicsItem 用于绘制鼠标轨迹
            # self.curve = PlotCurveItem(pen=QPen(Qt.red,0.05))
            # self.current_plot.addItem(self.curve)



    def on_mouse_move(self,pos):
        if self.is_drawing:

            # 将场景坐标转换为视图坐标
            mouse_point =self.view_box.mapSceneToView(pos)
            x, y = mouse_point.x(), mouse_point.y()
            # 记录轨迹数据
            self.x_data.append(x)
            self.y_data.append(y)

            # 更新绘图
            self.curve.setData(self.x_data, self.y_data)


    def _mouseMoveEvent(self, event):
        """当鼠标移动时，如果按住左键，则绘制鼠标的轨迹"""
        if self.is_drawing:
            pos = event.scenePos()
            # 将场景坐标转换为视图坐标
            mouse_point =self.view_box.mapSceneToView(pos)
            x, y = mouse_point.x(), mouse_point.y()
            # 记录轨迹数据
            self.x_data.append(x)
            self.y_data.append(y)

            # 更新绘图
            self.curve.setData(self.x_data, self.y_data)
        else:
            return ViewBox.mousePressEvent(self.view_box,event)

    def _mouseReleaseEvent(self, event):
        """鼠标松开时停止绘制，并标记轨迹内的散点"""
        if self.mode !=_Mode.PEN:
            return ViewBox.mousePressEvent(self.view_box,event)



        if event.button() == Qt.MouseButton.LeftButton  or event.button() == Qt.MouseButton.RightButton:
            self.is_drawing = False
            reverse=event.button() == Qt.MouseButton.RightButton
            self.current_plot.removeItem(self.curve)
            # 创建鼠标轨迹的多边形
            if len(self.x_data)>2:

                self.graph_widget.select_point_from_polygon(np.column_stack((self.x_data, self.y_data)),reverse)
            else:


                # 右键的话  选中单个点
                pass
                pos = event.scenePos()
                mouse_point = self.view_box.mapSceneToView(pos)

                x = mouse_point.x()
                self.graph_widget.select_point(mouse_point,reverse)
            return




#!/usr/bin/env python 
# -*- coding: utf-8 -*-
# @Time    : 2024/10/20 22:22
# @Author  : 兵
# @email    : 1747193328@qq.com

import numpy as np
from PySide6.QtCore import Signal
from pyqtgraph import mkPen, ScatterPlotItem, TextItem

from .canvas import CustomGraphicsLayoutWidget
from .. import MessageManager
from ..io import NepTrainResultData
from ..types import Brushes


class NepResultGraphicsLayoutWidget(CustomGraphicsLayoutWidget):
    structureIndexChanged=Signal(int)
    def __init__(self,parent=None):
        super().__init__(parent)
        self._parent=parent
        self.dataset=None


    def set_dataset(self,dataset):
        self.dataset:NepTrainResultData=dataset
        self.subplot(2,3)
        self.plot_all()

    def convert_pos(self,plot,pos):
        view_range = plot.viewRange()
        x_range = view_range[0]  # x轴范围 [xmin, xmax]
        y_range = view_range[1]  # y轴范围 [ymin, ymax]

        # 将百分比位置转换为坐标
        x_percent = pos[0] # 50% 对应 x 轴中间
        y_percent =  pos[1]  # 20% 对应 y 轴上的某个位置

        x_pos = x_range[0] + x_percent * (x_range[1] - x_range[0])  # 根据百分比计算实际位置
        y_pos = y_range[0] + y_percent * (y_range[1] - y_range[0])  # 根据百分比计算实际位置
        return x_pos,y_pos

    def get_current_dataset(self):
        if self.current_plot is None:
            return None
        plot_index = self.axes_list.index(self.current_plot)
        return self.dataset.dataset[plot_index]

    def plot_all(self):
        self.dataset.select_index.clear()
        _pen = mkPen(None)
        # import time
        # start = time.time()
        for index,_dataset in enumerate(self.dataset.dataset):
            plot=self.axes_list[index]
            plot.clear()
            if _dataset.title not in ["descriptor"]:
                plot.addLine(angle=45, pos=(0.5, 0.5), pen=mkPen('r', width=2))

            # else:
            plot.setTitle(_dataset.title)

            scatter = ScatterPlotItem(_dataset.x,_dataset.y,data=_dataset.structure_index,
                                      brush=_dataset.normal_color, pen=None, symbol='o',
                                     )


            scatter.sigClicked.connect(self.item_clicked)
            plot.scatter=scatter
            # plot.setDownsampling(auto=False )
            plot.addItem(scatter)
            if _dataset.title not in ["descriptor"]:

                pos=self.convert_pos(plot,(0 ,1))
                text=f"rmse: {_dataset.get_formart_rmse()}"
                text_item = TextItem(text=text ,color=(231, 63, 50))
                text_item.setPos(*pos)
                plot.addItem(text_item)

            # plot.autoRange()
        # print(time.time()-start)
        #5.67748498916626
    def item_clicked(self,scatter_item,items,event):

        if items.any():
            item=items[0]

            self.structureIndexChanged.emit(item.data())

    def delete(self):
        if self.dataset is not None and self.dataset.select_index:

            self.dataset.delete_selected()
            self.plot_all()


    def select_point_from_polygon(self,polygon_xy,reverse ):
        index=self.is_point_in_polygon(np.column_stack([self.current_plot.scatter.data["x"],self.current_plot.scatter.data["y"]]),polygon_xy)
        index = np.where(index)[0]
        select_index=self.current_plot.scatter.data[index]["data"].tolist()
        self.select_index(select_index,reverse)


    def select_point(self,pos,reverse):
        items=self.current_plot.scatter.pointsAt(pos)
        if len(items):

            item=items[0]

            index=item.index()
            structure_index =item.data()
            self.select_index(structure_index,reverse)

    def select_index(self,structure_index,reverse):
        if isinstance(structure_index,int):
            structure_index=[structure_index]
        elif isinstance(structure_index,np.ndarray):
            structure_index=structure_index.tolist()

        if not structure_index:
            return
        if reverse:
            self.dataset.uncheck(structure_index)
            self.update_axes_color(structure_index, Brushes.BlueBrush)

        else:

            self.dataset.select(structure_index)

            self.update_axes_color(structure_index, Brushes.RedBrush)


    def update_axes_color(self,structure_index,color=Brushes.RedBrush):


        for i,plot in enumerate(self.axes_list):

            if not hasattr(plot,"scatter"):
                continue
            structure_index_set= set(structure_index)
            index_list = [i for i, val in enumerate(plot.scatter.data["data"]) if val in structure_index_set]

            plot.scatter.data["brush"][index_list]=   color
            plot.scatter.data['sourceRect'][index_list] = (0, 0, 0, 0)


            plot.scatter.updateSpots(  )





    def revoke(self):
        """
        如果有删除的结构  撤销上一次删除的
        :return:
        """
        if self.dataset.is_revoke:
            self.dataset.revoke()
            self.plot_all()

        else:
            MessageManager.send_info_message("No undoable deletion!")


#!/usr/bin/env python 
# -*- coding: utf-8 -*-
# @Time    : 2024/10/24 14:22
# @Author  : 兵
# @email    : 1747193328@qq.com
"""Pymatgen package configuration."""

from __future__ import annotations

import platform
import sys


from setuptools import Extension, setup
import pybind11
import setuptools
from pybind11.setup_helpers import Pybind11Extension
from setuptools import find_packages, setup
from setuptools.command.build_ext import build_ext

# 获取 pybind11 的 include 路径
pybind11_include = pybind11.get_include()

# 定义扩展模块
ext_modules = [
    Extension(
        "nep_cpu",  # 模块名
        ["src/nep_cpu/nep_bindings.cpp"],  # 源文件
        include_dirs=[
            pybind11_include,
            # "src/nep_cpu"
        ],
        extra_compile_args=["-O3", "-Wall", "-std=c++17"],  # 编译选项
        extra_link_args=[],
        language="c++",  # 指定语言为 C++
    ),
]

# 自定义 build_ext 命令，确保兼容性
class BuildExt(build_ext):
    def build_extensions(self):
        # 设置编译器标准为 C++17
        ct = self.compiler.compiler_type
        opts = [ ]
        for ext in self.extensions:
            ext.extra_compile_args = opts + ext.extra_compile_args
        build_ext.build_extensions(self)

is_win_64 = sys.platform.startswith("win") and platform.machine().endswith("64")
extra_link_args = ["-Wl,--allow-multiple-definition"] if is_win_64 else []

setup(
    author="Chen Cheng bing",
cmdclass={'build_ext': BuildExt},
    # include_dirs=[np.get_include()],
ext_modules=ext_modules,
zip_safe=False,
)


#!/usr/bin/env python 
# -*- coding: utf-8 -*-
# @Time    : 2024/11/21 14:22
# @Author  : 兵
# @email    : 1747193328@qq.com
import contextlib
import os

import numpy as np
from PySide6.QtCore import QObject

from nep_cpu import NEP3

class Nep3Calculator(QObject):

    def __init__(self, model_file="nep.txt"):
        super().__init__()
        if not isinstance(model_file, str):
            model_file=str(model_file,encoding="utf-8")
        with open(os.devnull, 'w') as devnull:
            with contextlib.redirect_stdout(devnull), contextlib.redirect_stderr(devnull):
                self.nep3 = NEP3(model_file)
        self.element_list=self.nep3.get_element_list()
        self.type_dict = {e: i for i, e in enumerate(self.element_list)}




    def get_descriptors(self,structure):
        symbols = structure.elements
        _type = [self.type_dict[k] for k in symbols]
        _box = structure.cell.transpose(1, 0).reshape(-1).tolist()

        _position = structure.positions.transpose(1, 0).reshape(-1).tolist()

        descriptor = self.nep3.find_descriptors(_type, _box, _position)

        descriptors_per_atom = np.array(descriptor).reshape(-1, len(structure)).T

        return descriptors_per_atom

if __name__ == '__main__':
    Nep3Calculator("../nep.txt")